#include "sgial.h"

static GtkWidget *configure_win = NULL, *vbox, *notebook;
static GtkWidget *dev_vbox, *adevice_frame, *adevice_box, *adevice;
static GtkWidget *buffer_frame, *buffer_vbox, *buffer_table;
static GtkWidget *buffer_size_box, *buffer_size_label, *buffer_size_spin; 
static GtkObject *buffer_size_adj;
static GtkWidget *bbox, *ok, *cancel;

static void configure_win_ok_cb(GtkWidget * w, gpointer data)
{
  alinfo.bufsize = (gint) GTK_ADJUSTMENT(buffer_size_adj)->value;

  cfgfile = xmms_cfg_open_file(filename);
  if (!cfgfile)
    cfgfile = xmms_cfg_new();
  
  xmms_cfg_write_string(cfgfile, "SGIAL", "output_device", 
			alinfo.current_device);
  xmms_cfg_write_int(cfgfile, "SGIAL", "buffer_size",
		     alinfo.bufsize);
  xmms_cfg_write_file(cfgfile, filename);
  xmms_cfg_free(cfgfile);

  gtk_widget_destroy(configure_win);
}

void scan_devices(gchar * type, GtkWidget * option_menu, GtkSignalFunc sigfunc)
{
  GtkWidget *menu, *item;
  ALpv q[1];
  char name[32];
  int i;

  menu = gtk_menu_new();

  for(i = 0; i < alinfo.device_count; i++) {
    q[0].param = AL_NAME;
    q[0].value.ptr = name;
    q[0].sizeIn = 32;
    alGetParams(alinfo.output_devices[i].i, q, 1);
    item = gtk_menu_item_new_with_label(name);
    gtk_signal_connect(GTK_OBJECT(item), "activate", sigfunc, (gpointer) 0);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
  }
  gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
}

/*
 * FIXME: fill in alinfo.output_device with string from option menu
 */
static void configure_win_audio_dev_cb(GtkWidget * widget, gint device)
{
  ALpv q[1];
  char name[32];

  q[0].param = AL_NAME;
  q[0].value.ptr = name;
  q[0].sizeIn = 32;
  alGetParams(alinfo.output_devices[device].i, q, 1);
  
  free(alinfo.current_device);
  alinfo.current_device = malloc(sizeof(char) * strlen(name));
  strcpy(alinfo.current_device, name);  

  cfgfile = xmms_cfg_open_file(filename);
  if (!cfgfile)
    cfgfile = xmms_cfg_new();
  
  xmms_cfg_write_string(cfgfile, "SGIAL", "output_device", 
			alinfo.current_device);
  xmms_cfg_write_int(cfgfile, "SGIAL", "buffer_size",
		     alinfo.bufsize);
  xmms_cfg_write_file(cfgfile, filename);
  xmms_cfg_free(cfgfile);
}

void sgial_configure(void)
{
	if (!configure_win)
	{
		configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
		gtk_signal_connect(GTK_OBJECT(configure_win), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &configure_win);
		gtk_window_set_title(GTK_WINDOW(configure_win), "Plugin configuration");
		gtk_window_set_policy(GTK_WINDOW(configure_win), FALSE, FALSE, FALSE);
		gtk_window_set_position(GTK_WINDOW(configure_win), GTK_WIN_POS_MOUSE);
		gtk_container_border_width(GTK_CONTAINER(configure_win), 10);

		vbox = gtk_vbox_new(FALSE, 10);
		gtk_container_add(GTK_CONTAINER(configure_win), vbox);

		notebook = gtk_notebook_new();
		gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);

		dev_vbox = gtk_vbox_new(FALSE, 5);
		gtk_container_set_border_width(GTK_CONTAINER(dev_vbox), 5);

		adevice_frame = gtk_frame_new("Audio device:");
		gtk_box_pack_start(GTK_BOX(dev_vbox), adevice_frame, FALSE, FALSE, 0);

		adevice_box = gtk_hbox_new(FALSE, 0);
		gtk_container_set_border_width(GTK_CONTAINER(adevice_box), 5);
		gtk_container_add(GTK_CONTAINER(adevice_frame), adevice_box);

		adevice = gtk_option_menu_new();
		gtk_box_pack_start(GTK_BOX(adevice_box), adevice, TRUE, TRUE, 0);
		scan_devices("Audio devices:", adevice, configure_win_audio_dev_cb);
		gtk_widget_show(adevice);
		gtk_widget_show(adevice_box);
		gtk_widget_show(adevice_frame);

		gtk_widget_show(dev_vbox);
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), dev_vbox, gtk_label_new("Devices"));

		buffer_frame = gtk_frame_new("Buffering:");
		gtk_container_set_border_width(GTK_CONTAINER(buffer_frame), 5);

		buffer_vbox = gtk_vbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(buffer_frame), buffer_vbox);

		buffer_table = gtk_table_new(2, 1, TRUE);
		gtk_container_set_border_width(GTK_CONTAINER(buffer_table), 5);
		gtk_box_pack_start(GTK_BOX(buffer_vbox), buffer_table, FALSE, FALSE, 0);

		buffer_size_box = gtk_hbox_new(FALSE, 5);
		gtk_table_attach_defaults(GTK_TABLE(buffer_table), buffer_size_box, 0, 1, 0, 1);
		buffer_size_label = gtk_label_new("Buffer size (frames):");
		gtk_box_pack_start(GTK_BOX(buffer_size_box), buffer_size_label, FALSE, FALSE, 0);
		gtk_widget_show(buffer_size_label);
		buffer_size_adj = gtk_adjustment_new(alinfo.bufsize, 5000, 500000, 100, 100, 100);
		buffer_size_spin = gtk_spin_button_new(GTK_ADJUSTMENT(buffer_size_adj), 8, 0);
		gtk_widget_set_usize(buffer_size_spin, 60, -1);
		gtk_box_pack_start(GTK_BOX(buffer_size_box), buffer_size_spin, FALSE, FALSE, 0);
		gtk_widget_show(buffer_size_spin);
		gtk_widget_show(buffer_size_box);

		gtk_widget_show(buffer_table);
		gtk_widget_show(buffer_vbox);
		gtk_widget_show(buffer_frame);
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), buffer_frame, gtk_label_new("Buffering"));

		gtk_widget_show(notebook);

		bbox = gtk_hbutton_box_new();
		gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
		gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
		gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);

		ok = gtk_button_new_with_label("Ok");
		gtk_signal_connect(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(configure_win_ok_cb), NULL);
		GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
		gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 0);
		gtk_widget_show(ok);
		gtk_widget_grab_default(ok);

		cancel = gtk_button_new_with_label("Cancel");
		gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(configure_win));
		GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
		gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 0);
		gtk_widget_show(cancel);

		gtk_widget_show(bbox);
		gtk_widget_show(vbox);
		gtk_widget_show(configure_win);
	}
	else
		gdk_window_raise(configure_win->window);
}
