#include "sgial.h"

/*
 * Flush the audio frames that haven't been played
 * and move to the specified time
 */
void sgial_flush(gint time)
{
  pinfo.output_time_offset = time;
  pinfo.written_frames = (gint) ((gfloat)(pinfo.written_frames * 1000) / 
				 (gfloat)(pinfo.rate));
  pinfo.output_frames = 0;
  alDiscardFrames(alinfo.port, alGetFilled(alinfo.port));
  return;
}

/*
 * Since we're not using our own buffer for output
 * this loses some of audio frames when called.  The solution
 * is to create an output loop that runs as a seperate
 * thread, which can output from an internal buffer.
 * Then this function can set an index to stop at in the
 * buffer so we don't lose any data.
 * FIXME: fix so we don't lose data
 */
void sgial_pause(short p)
{
  if (alinfo.port && p) {
    alDiscardFrames(alinfo.port, alGetFilled(alinfo.port));
  }

  pinfo.paused = p;
  return;
}

/*
 * Get the number of available bytes in the buffer
 * (but if we're paused don't accept more).
 */
gint sgial_free(void)
{
  if(!pinfo.paused)
    return alGetFillable(alinfo.port) * alinfo.frame_size;
  return 0;
}

/*
 * Are we playing anything?  This refers to if we are
 * actually sending anything to the audio buffer at the
 * moment.  This is typically called after a song finishes.
 * XMMS wants to make sure we're not still playing before
 * it sends more data.  Since we can always put more in the
 * buffer, we always tell XMMS that we're not playing.
 */
gint sgial_playing(void)
{
  pinfo.playing = FALSE;
  return pinfo.playing;
}

/*
 * Return the time offset (in ms) of what's coming out
 * the speaker.
 */
gint sgial_get_output_time(void)
{
  gint time, latency = 0;

  if(!pinfo.playing) {
    return 0;
  }

  /*
   * If we're paused, don't count latency
   */
  if(!pinfo.paused)
    latency = alGetFilled(alinfo.port);
  if(latency < 0)
    latency = 0;

  time = pinfo.output_time_offset + 
    (gint) ((double)((pinfo.output_frames - latency) * 1000) /
	    (double)pinfo.rate);
  if (time < 0)
    time = 0;
  
  return time;
}

/*
 * Return the time offset (in ms) of what
 * we just sent to the audio buffer.
 */
gint sgial_get_written_time(void)
{
  if(!pinfo.playing) {
    return 0;
  }
  return (gint) ((double)(pinfo.written_frames * 1000) / 
		 (double)pinfo.rate);
}

/*
 * Calculate the latency based on the buffer size
 * FIXME: write this function
 */
gint get_latency(void)
{
  gint amount;
  amount = (44100 * alinfo.bufsize) / pinfo.rate;
  return amount;
}
