#include "sgial.h"

/*
 * Open the audio port
 *
 * FIXME: add switch for rate variations
 *       convert between different formats
 */
gint sgial_open(AFormat sampsize, gint rate, gint nch)
{
  ALpv params[1];

  /*
   * Make sure we get a valid config handle
   */
  if ( (alinfo.config = alNewConfig()) == 0) {
    fprintf(stderr,"open: alNewConfig : %s\n",alGetErrorString(oserror()));
    return 0;
  }

  /*
   * Calculate the bitrate
   */
  pinfo.bitrate = rate * nch;
  pinfo.rate = rate;

  /*
   * Hopefully we're using a twos complement sample format
   * (or this is gonna sound weird).  Other formats for
   * dmedia include AL_SAMPFMT_FLOAT and AL_SAMPFMT_DOUBLE
   */
  pinfo.format = AL_SAMPFMT_TWOSCOMP;
  if(alSetSampFmt(alinfo.config, pinfo.format) < 0) {
    fprintf(stderr,"open: alSetSampFmt : %s\n",alGetErrorString(oserror()));
  }

  /*
   * Determine and set sample size
   */
  switch (sampsize)
    {
    case FMT_U8:
    case FMT_S8:
      pinfo.width = AL_SAMPLE_8;
      break;
    case FMT_U16_LE:
    case FMT_U16_BE:
    case FMT_U16_NE:
    case FMT_S16_LE:
    case FMT_S16_BE:
    case FMT_S16_NE:
      pinfo.bitrate *= 2;
      pinfo.width = AL_SAMPLE_16;
      break;
    }
  if(alSetWidth(alinfo.config, pinfo.width) < 0) {
    fprintf(stderr,"open: alSetWidth : %s\n",alGetErrorString(oserror()));
  }
 
  /*
   * Are we stereo or mono?
   */
  if(nch == 1) {
    if(alSetChannels(alinfo.config, AL_MONO) < 0) {
      fprintf(stderr,"open: alSetChannels : %s\n",alGetErrorString(oserror()));
    }
  }
  else {
    if(alSetChannels(alinfo.config, AL_STEREO) < 0) {
      fprintf(stderr,"open: alSetChannels : %s\n",alGetErrorString(oserror()));
    }
  }

  /*
   * Number of bytes per audio frame
   */
  alinfo.frame_size = pinfo.width * nch;

  /*
   * Set the queue size to the user configured buffer size
   */
  if(alSetQueueSize(alinfo.config, alinfo.bufsize) < 0) {
    fprintf(stderr,"open: alSetQueueSize : %s\n",alGetErrorString(oserror()));
  }

  /*
   * Based on the buffer size, get the latency
  pinfo.latency = sgial_get_latency();
   */
  
  /*
   * Set the device in the config structure
   */
  if (alSetDevice(alinfo.config, alinfo.current_dev) < 0) {
    fprintf(stderr,"open: alSetDevice : %s\n",alGetErrorString(oserror()));
    return 0;
  }

  /*
   * Setup the default output rate
   */
  params[0].param = AL_RATE;
  params[0].sizeIn = 1;
  params[0].value.ll = alDoubleToFixed((double)AL_RATE_44100);
  if(alSetParams(alinfo.current_dev, params, 1) < 0) {
    fprintf(stderr,"open: alSetParams : %s\n",alGetErrorString(oserror()));
  }
    
  /*
   * The moment of truth, opening the port for writing
   */
  if((alinfo.port = alOpenPort("XMMS", "w", alinfo.config)) == NULL) {
    fprintf(stderr, "open: alOpenPort : %s", alGetErrorString(oserror()));
    return 0;
  }

  pinfo.playing = TRUE;
  pinfo.output_time_offset = 0;
  pinfo.written_frames = 0;
  pinfo.output_frames = 0;
  return 1;
}

/*
 * Close the port, flush the queue, write to the config file
 */
void sgial_close(void)
{
  if (alinfo.port) {
    alDiscardFrames(alinfo.port, alGetFilled(alinfo.port));
    alClosePort(alinfo.port);
    alFreeConfig(alinfo.config);
  }

  pinfo.playing = FALSE;
  return;
}
