#include "sgial.h"

/*
 * Get the volume 
 */
void sgial_get_volume(int *r, int *l)
{
  ALpv pvs[1];
  ALfixed gain[2];
  
  /*
   * If we haven't opened a port yet, use the default
   */
  if(alinfo.current_dev == 0) {
    alinfo.current_dev = AL_DEFAULT_OUTPUT;
  }

  pvs[0].param = AL_GAIN;
  pvs[0].value.ptr = gain;
  pvs[0].sizeIn = 2;

  if (alGetParams(alinfo.current_dev, pvs, 1) < 0) {
    fprintf(stderr, "get_volume: alGetParams failed: %s\n", 
	   alGetErrorString(oserror()));
  }

  alinfo.rightgain = alFixedToDouble(gain[0]);  
  alinfo.leftgain = alFixedToDouble(gain[1]);

  /*
   * See above on how we calculate volume
   */
  *l = (int)(((alinfo.leftgain + alinfo.adjustment) / alinfo.gainrange) * 100);
  *r = (int)(((alinfo.rightgain + alinfo.adjustment) / alinfo.gainrange) * 100);

  return;
}

/*
 * Set the volume to r, l in %
 */
void sgial_set_volume(int r, int l)
{
  double lpct, rpct;
  double newleft, newright;
  ALpv vol[1];
  ALfixed newvols[2];

  /*
   * If we haven't opened a port yet, use the default
   */
  if(alinfo.current_dev == 0) {
    alinfo.current_dev = AL_DEFAULT_OUTPUT;
  }

  /*
   * Calculate the gain (in decibels) based on the % passed in
   */
  lpct = l / 100.0;
  rpct = r / 100.0;
  newleft = (lpct * alinfo.gainrange) - alinfo.adjustment;
  newright = (rpct * alinfo.gainrange) - alinfo.adjustment;

  /*
   * Set the gain on the default output device or
   * mute it if a zero volume is selected.  Another
   * way of doing this would be to use AL_NEG_INFINITY
   * as the gain, but not all devices support it, so we
   * unmute it every time instead.
   */
  if(lpct > 0 || rpct > 0) {
    /*
     * First unmute it
     */
    vol[0].param = AL_MUTE;
    vol[0].value.i = 0;
    if (alSetParams(alinfo.current_dev, vol, 1) < 0) {
      fprintf(stderr, "set_volume: alSetParams failed: %s\n", 
	      alGetErrorString(oserror()));
    }
    
    /* 
     * Now set the gain
     */
    vol[0].param = AL_GAIN;
    vol[0].value.ptr = newvols;
    vol[0].sizeIn = 2;
    newvols[0] = alDoubleToFixed(newright);
    newvols[1] = alDoubleToFixed(newleft);
  }
  else {
    /*
     * We got a zero %, so we mute it
     */
    vol[0].param = AL_MUTE;
    vol[0].value.i = 1;
  }

  if (alSetParams(alinfo.current_dev, vol, 1) < 0) {
    fprintf(stderr, "set_volume: alSetParams failed: %s\n", 
	   alGetErrorString(oserror()));
  }

  alinfo.leftgain = l;
  alinfo.rightgain = r;

  return;
}

